# SloMoMod release 3 by David Bowland
# ./addons/eventscripts/slomomod/slomomod.py

# >>> To configure this addon please see slomomod.cfg <<<

"""
Controls server timescale for slow motion or speeding up time when only bots are playing. Also provides bullet tracers. Requires only ES 2.0+
"""


import es
import gamethread
import os.path
import random


info = es.AddonInfo()
info.name = 'SloMoMod'
info.version = '3'
info.url = 'http://addons.eventscripts.com/addons/view/slomomod'
info.basename = 'slomomod'
info.author = 'SuperDave'


bool_bomb_planted = False
bool_round_over = False
int_tracers_status = 0
int_last_sv_cheats = 0
str_current_trigger = ''
dict_triggers = {}
dict_player_count = {'human':0, 'bot':0}
dict_tracer_data = {}

tuple_weapon_tags = ('#primary', '#rifle', '#shotgun', '#smg', '#sniper', '#secondary', '#pistol')
dict_weapons = {'deagle':('#pistol', '#secondary'), 'ak47':('#rifle', '#primary'), 'scout':('#sniper', '#rifle', '#primary'), 'aug':('#rifle', '#primary'), 'g3sg1':('#sniper', '#rifle', '#primary'), 'galil':('#rifle', '#primary'), 'famas':('#rifle', '#primary'), 'm4a1':('#rifle', '#primary'), 'sg552':('#rifle', '#primary'), 'sg550':('#sniper', '#rifle', '#primary'), 'm249':('#rifle', '#primary'), 'awp':('#sniper', '#rifle', '#primary'), 'tmp':('#smg', '#primary'), 'mp5navy':('#smg', '#primary'), 'glock':('#pistol', '#secondary'), 'elite':('#pistol', '#secondary'), 'm3':('#shotgun', '#primary'), 'xm1014':('#shotgun', '#primary'), 'usp':('#pistol', '#secondary'), 'mac10':('#smg', '#primary'), 'ump45':('#smg', '#primary'), 'p228':('#pistol', '#secondary'), 'fiveseven':('#pistol', '#secondary'), 'p90':('#smg', '#primary'), 'hegrenade':(), 'knife':(), 'team':(), 'team_suicide':(), 'other':()}
dict_events = {'slomomod_activate':{'trigger':'string', 'duration':'float', 'timescale':'float'}, 'slomomod_deactivate':{'trigger':'string'}}
dict_options = {'slomomod_cheat_notify':[0, '0 = sv_cheats notification will only be suppressed when changed by SloMoMod, 1 = sv_cheats notification will be suppressed for the life of the server'], 'slomomod_bomb_planted':[1, '0 = no slow motion when the bomb is planted, 1 = only end-of-round slow motion when the bomb is planted, 2 = full slow motion when the bomb is planted'], 'slomomod_sound_volume':[0.8, 'Default volume to play sounds from 0 to 1 inclusive'], 'slomomod_tracer':[1, '0 = no bullet tracers, 1 = show bullet tracers only during slow motion, 2 = always show bullet tracers'], 'slomomod_tracer_kill':[0,' 0 = no artificial kill trace, 1 = create an artificial tracer from attackers to victims only when bullet tracers are in effect, 2 = create an artificial tracer from attackers to victims every kill'], 'slomomod_tracer_everyimpact':[1, '0 = only one tracer per weapon fire is allowed, 1 = trace all bullet impacts'], 'slomomod_tracer_width_start':[3, 'Width of tracers at the player'], 'slomomod_tracer_width_end':[1, 'Width of tracers at bullet impact'], 'slomomod_tracer_noise_min':[0, 'Minumim amount of noise (vibration) in tracers'], 'slomomod_tracer_noise_max':[3, 'Maximum amount of noise (vibration) in tracers'], 'slomomod_tracer_delay_display':[3, 'Number of seconds tracers are displayed'], 'slomomod_tracer_delay_fade':[1.5, 'Number of seconds tracers take to fade out'], 'slomomod_tracer_brightness':[255, 'Brightness of tracers from 0 to 255 inclusive'], 'slomomod_tracer_color_t_red':[200, 'Intensity of red in Terrorist tracers from 0 to 255 inclusive'], 'slomomod_tracer_color_t_green':[0, 'Intensity of green in Terrorist tracers from 0 to 255 inclusive'], 'slomomod_tracer_color_t_blue':[0, 'Intensity of blue in Terrorist tracers from 0 to 255 inclusive'], 'slomomod_tracer_color_ct_red':[0, 'Intensity of red in Counter-Terrorist tracers from 0 to 255 inclusive'], 'slomomod_tracer_color_ct_green':[0, 'Intensity of green in Counter-Terrorist tracers from 0 to 255 inclusive'], 'slomomod_tracer_color_ct_blue':[200, 'Intensity of blue in Counter-Terrorist tracers from 0 to 255 inclusive'], 'slomomod_tracer_model':['sprites/laser.vmt', 'Model of bullet tracers relative to ./materials/ with .vmt extension--model will not be downloaded by clients']}


def load():
   """
   Creates server commands
   Checks to make sure critical server variables are created by the config
   Calls load_events
   Calls round_start
   """
   if not es.exists('command', 'slomomod_add'):
      es.regcmd('slomomod_add', 'slomomod/trigger_cmd', 'slomomod_add <trigger> <duration> <timescale> [interruptible] [sound] [force sound download] [volume]\nServer command for defining when timescale should be altered')
   if not es.exists('command', 'slomomod_remove'):
      es.regcmd('slomomod_remove', 'slomomod/trigger_cmd', 'slomomod_remove <trigger>\nServer command for removing slow motion triggers')
   if not es.exists('command', 'slomomod_start'):
      es.regcmd('slomomod_start', 'slomomod/start_cmd', 'slomomod_start <duration> <timescale> [interruptible] [sound] [force sound download] [volume]\nServer command for starting custom slow motion')
   if not es.exists('command', 'slomomod_end'):
      es.regcmd('slomomod_end', 'slomomod/end_cmd', 'slomomod_end [force end]\nServer command for restoring timescale to normal')

   for str_option in dict_options:
      es.ServerVar(str_option, dict_options[str_option][0], dict_options[str_option][1])
   if os.path.isfile(es.getAddonPath('slomomod') + '/slomomod.cfg'):
      es.server.cmd('es_xmexec ../addons/eventscripts/slomomod/slomomod.cfg')
   else:
      es.dbgmsg(0, 'SloMoMod: Unable to load slomomod.cfg! Please ensure it is in the ./slomomod/ directory.')

   load_events()

   round_start({})

   if int(es.ServerVar('slomomod_cheat_notify')):
      es.flags('remove', 'notify', 'sv_cheats')


def es_map_start(event_var):
   """
   Calls load_events
   Marks sounds for download
   """
   load_events()

   for str_trigger in dict_triggers:
      dict_current_trigger = dict_triggers[str_trigger]
      if dict_current_trigger['downloadable']:
         es.stringtable('downloadables', 'sound/' + dict_current_trigger['sound'])


def round_start(event_var):
   """
   Sets bool_bomb_planted to whether or not the bomb has been planted, sets bool_round_over to false
   Counts living bots and humans
   Resets bullet tracers
   Returns timescale to normal and triggers only_bots if necessary
   """
   global bool_bomb_planted
   global bool_round_over
   global dict_player_count

   bool_bomb_planted = es.getentityindex('planted_c4') > -1
   bool_round_over = False

   dict_player_count['human'] = 0
   dict_player_count['bot'] = 0

   for int_userid in es.getUseridList():
      if es.getplayerteam(int_userid) > 1 and not es.getplayerprop(int_userid, 'CBasePlayer.pl.deadflag'):
         if es.getplayersteamid(int_userid) == 'BOT':
            dict_player_count['bot'] += 1
         else:
            dict_player_count['human'] += 1

   tracers_off()
   if es.ServerVar('slomomod_tracer') == 2:
      tracers_on()

   if str_current_trigger:
      end_slomo()
   elif dict_player_count['bot'] and not dict_player_count['human']:
      trigger_slomo('only_bots')


def round_end(event_var):
   """
   Sets bool_round_over
   Triggers roundend timescale and triggers roundend_timelimit if round draw, target not destroyed, or hostages not rescued
   """
   global bool_round_over

   bool_round_over = True

   list_triggers = []
   if event_var['reason'] in ('10', '12', '13'):
      list_triggers.append('roundend_timelimit')

   list_triggers.append('roundend')
   trigger_slomo(list_triggers)


def player_spawn(event_var):
   """
   Adds to the count of bots or humans
   Ends only_bots trigger if the spawning player is human
   """
   global dict_player_count

   if int(event_var['es_userteam']) > 1:
      if event_var['es_steamid'] == 'BOT':
         dict_player_count['bot'] += 1
      else:
         dict_player_count['human'] += 1
         if str_current_trigger == 'only_bots':
            end_slomo()


def player_death(event_var):
   """
   Triggers the timescale applicable to this death
   Calls weapon_fire for the attacking player to get the player's position in case bullet tracers were called into effect by the timescale trigger
   """
   global bool_round_over
   global int_tracers_status
   global dict_player_count

   list_triggers = []
   int_headshot = int(event_var['headshot'])
   str_team = event_var['es_userteam']
   str_weapon = event_var['weapon']
   str_userid = event_var['userid']
   str_attacker = event_var['attacker']
   bool_human_user = event_var['es_steamid'] <> 'BOT'
   bool_human_attacker = event_var['es_attackersteamid'] <> 'BOT' and int(str_attacker)
   if str_userid == str_attacker:
      str_weapon = 'team_suicide'
   elif str_team == event_var['es_attackerteam']:
      str_weapon = 'team'
   elif not dict_weapons.has_key(str_weapon):
      str_weapon = 'other'

   if es.getlivingplayercount() == 1:
      list_triggers.append('death_onestanding')

   if es.getlivingplayercount(str_team) == 0:
      list_triggers.append('death_lastteam')
      if not bool_bomb_planted or str_team == '3':
         bool_round_over = True

   if bool_human_user:
      dict_player_count['human'] -= 1
      if not dict_player_count['human']:
         list_triggers.append('death_lasthuman')
   else:
      dict_player_count['bot'] -= 1

   if int_headshot:
      if bool_human_user:
         list_triggers.append('humandeath_%s_headshot' % str_weapon)
      if bool_human_attacker:
         list_triggers.append('humankill_%s_headshot' % str_weapon)
      list_triggers.append('death_%s_headshot' % str_weapon)
      for str_tag in dict_weapons[str_weapon]:
         if bool_human_user:
            list_triggers.append('humandeath_%s_headshot' % str_tag)
         if bool_human_attacker:
            list_triggers.append('humankill_%s_headshot' % str_tag)
         list_triggers.append('death_%s_headshot' % str_tag)

   if bool_human_user:
      list_triggers.append('humandeath_%s' % str_weapon)
   if bool_human_attacker:
      list_triggers.append('humankill_%s' % str_weapon)
   list_triggers.append('death_%s' % str_weapon)
   for str_tag in dict_weapons[str_weapon]:
      if bool_human_user:
         list_triggers.append('humandeath_%s' % str_tag)
      if bool_human_attacker:
         list_triggers.append('humankill_%s' % str_tag)
      list_triggers.append('death_%s' % str_tag)

   if int_headshot:
      if bool_human_user:
         list_triggers.append('humandeath_headshot')
      if bool_human_attacker:
         list_triggers.append('humankill_headshot')
      list_triggers.append('death_headshot')

   if bool_human_user:
      list_triggers.append('humandeath')
   if bool_human_attacker:
      list_triggers.append('humankill')
   list_triggers.append('death')

   if dict_player_count['bot'] and not dict_player_count['human']:
      list_triggers.append('only_bots')

   trigger_slomo(list_triggers)

   if int(str_attacker):
      int_kill_trace = int(es.ServerVar('slomomod_tracer_kill'))
      if int_tracers_status:
         weapon_fire({'userid':str_attacker, 'weapon':str_weapon})
      if int_kill_trace == 1 and int_tracers_status or int_kill_trace > 1:
         int_temp_tracers_status = int_tracers_status
         int_tracers_status = 1
         weapon_fire({'userid':str_attacker, 'weapon':str_weapon})
         list_player_location = list(es.getplayerlocation(str_userid))
         list_player_location[2] += -10 + -15 * (es.getplayerprop(str_userid, 'CBasePlayer.localdata.m_Local.m_bDucked') % 2) + 15 * int_headshot
         bullet_impact({'userid':str_attacker, 'es_userteam':event_var['es_attackerteam'], 'x':str(list_player_location[0]), 'y':str(list_player_location[1]), 'z':str(list_player_location[2])})
         int_tracers_status = int_temp_tracers_status


def bomb_planted(event_var):
   """Resets timescale to normal if the bomb is planted"""
   global bool_bomb_planted

   bool_bomb_planted = True

   if str_current_trigger:
      int_bomb_option = int(es.ServerVar('slomomod_bomb_planted'))
      if int_bomb_option == 0 or (int_bomb_option == 1 and not bool_round_over):
         end_slomo()


def bomb_exploded(event_var):
   """Triggers bomb_exploded timescale"""
   global bool_round_over

   bool_round_over = True
   trigger_slomo('bomb_exploded')


def bomb_defused(event_var):
   """Triggers bomb_defused timescale"""
   global bool_round_over

   bool_round_over = True
   trigger_slomo('bomb_defused')


def hostage_killed(event_var):
   """Triggers hostage_killed timescale"""
   trigger_slomo('hostage_killed')


def hostage_rescued(event_var):
   """Triggers hostage_rescued timescale"""
   trigger_slomo('hostage_rescued')


def hostage_rescued_all(event_var):
   """Triggers all_hostages_rescued timescale"""
   global bool_round_over

   bool_round_over = True
   trigger_slomo('all_hostages_rescued')


def weapon_fire(event_var):
   """Captures the player's position for use with bullet tracers"""
   global dict_tracer_data

   if int_tracers_status:
      str_weapon = event_var['weapon']
      if dict_weapons.has_key(str_weapon):
         if '#primary' in dict_weapons[str_weapon] or '#secondary' in dict_weapons[str_weapon]:
            str_userid = event_var['userid']
            list_player_location = list(es.getplayerlocation(str_userid))
            list_player_location[2] += 60 - 15 * (es.getplayerprop(str_userid, 'CBasePlayer.localdata.m_Local.m_bDucked') % 2)
            dict_tracer_data[str_userid] = list_player_location


def bullet_impact(event_var):
   """Generates bullet tracer based on player's position and the position of the bullet impact"""
   global dict_tracer_data

   if int_tracers_status:
      str_userid = event_var['userid']
      if dict_tracer_data.has_key(str_userid):
         int_model_index = es.precachemodel(str(es.ServerVar('slomomod_tracer_model')))
         list_player_location = dict_tracer_data[str_userid]
         str_team_name = {'2':'t', '3':'ct'}[event_var['es_userteam']]
         es.effect('beam', '%s,%s,%s' % (event_var['x'], event_var['y'], event_var['z']), '%s,%s,%s' % (list_player_location[0], list_player_location[1], list_player_location[2]), int_model_index, int_model_index, 0, 255, es.ServerVar('slomomod_tracer_delay_display'), es.ServerVar('slomomod_tracer_width_end'), es.ServerVar('slomomod_tracer_width_start'), es.ServerVar('slomomod_tracer_delay_fade'), random.uniform(float(es.ServerVar('slomomod_tracer_noise_min')), float(es.ServerVar('slomomod_tracer_noise_max'))), es.ServerVar('slomomod_tracer_color_' + str_team_name + '_red'), es.ServerVar('slomomod_tracer_color_' + str_team_name + '_green'), es.ServerVar('slomomod_tracer_color_' + str_team_name + '_blue'), es.ServerVar('slomomod_tracer_brightness'), 10)

         if not int(es.ServerVar('slomomod_tracer_everyimpact')):
            del dict_tracer_data[str_userid]


def unload():
   """
   Resets timescale to normal
   Removes bullet tracers (and eventscripts_noisy)
   """
   if str_current_trigger:
      end_slomo(bool_check_bots=False)

   tracers_off()

   for str_option in dict_options:
      es.ServerVar(str_option).set(0)


def trigger_slomo(arg_triggers):
   """
   Stops method if bomb is planted or current trigger cannot be interrupted
   Converts the argument to an object that can be iterated
   Calls end_slomo and start_slomo for the first valid trigger found
   """
   if bool_bomb_planted:
      int_bomb_option = int(es.ServerVar('slomomod_bomb_planted'))
      if int_bomb_option == 0 or (int_bomb_option == 1 and not bool_round_over):
         return

   if str_current_trigger:
      if not dict_triggers[str_current_trigger]['interruptible']:
         return

   if isinstance(arg_triggers, str):
      arg_triggers = (arg_triggers,)

   for str_trigger in arg_triggers:
      if dict_triggers.has_key(str_trigger):
         if str_current_trigger:
            end_slomo(bool_check_bots=False)

         start_slomo(str_trigger)
         break


def start_slomo(str_trigger):
   """
   Plays the sound for all players
   Sets sv_cheats and host_timescale
   Files the slomomod_activate event
   Turns on tracers
   Starts a delay to call end_slomo
   """
   global str_current_trigger
   global int_last_sv_cheats

   str_current_trigger = str_trigger
   dict_current_trigger = dict_triggers[str_trigger]
   int_last_sv_cheats = int(es.ServerVar('sv_cheats'))
   float_duration = dict_current_trigger['duration']
   float_timescale = dict_current_trigger['timescale']
   str_sound = dict_current_trigger['sound']
   float_volume = dict_current_trigger['volume']
   if float_volume == -1:
      float_volume = float(es.ServerVar('slomomod_sound_volume'))

   if str_sound:
      for int_userid in es.getUseridList():
         es.playsound(int_userid, str_sound, float_volume)

   if not int_last_sv_cheats:
      es.flags('remove', 'notify', 'sv_cheats')
      es.server.cmd('sv_cheats 1')
   es.server.cmd('host_timescale %s' % float_timescale)

   es.event('initialize', 'slomomod_activate')
   es.event('setstring', 'slomomod_activate', 'trigger', str_current_trigger)
   es.event('setfloat', 'slomomod_activate', 'duration', float_duration)
   es.event('setfloat', 'slomomod_activate', 'timescale', float_timescale)
   es.event('fire', 'slomomod_activate')

   if str_current_trigger <> 'only_bots':
      if int(es.ServerVar('slomomod_tracer')):
         tracers_on()
      gamethread.delayedname(float_duration, 'slomomod_slomo', end_slomo, False)


def end_slomo(bool_interrupted=True, bool_check_bots=True):
   """
   Cancels the delay
   Stops the sound for all players
   Fires the slomomod_deactivate event
   Resets host_timescale, sv_cheats, and sv_cheats notify flag
   Turns off tracers
   Calls trigger_slomo if only bots are left
   """
   global str_current_trigger

   if bool_interrupted and str_current_trigger <> 'only_bots':
      gamethread.cancelDelayed('slomomod_slomo')

   str_sound = dict_triggers[str_current_trigger]['sound']
   if str_sound:
      for int_userid in es.getUseridList():
         es.stopsound(int_userid, str_sound)

   es.event('initialize', 'slomomod_deactivate')
   es.event('setstring', 'slomomod_deactivate', 'trigger', str_current_trigger)
   es.event('fire', 'slomomod_deactivate')

   str_current_trigger = ''
   es.server.cmd('host_timescale 1.0')

   if not int_last_sv_cheats:
      es.server.cmd('sv_cheats 0')
      if not int(es.ServerVar('slomomod_cheat_notify')):
         es.flags('add', 'notify', 'sv_cheats')

   if es.ServerVar('slomomod_tracer') <> 2:
      tracers_off()

   if bool_check_bots and dict_player_count['bot'] and not dict_player_count['human']:
      trigger_slomo('only_bots')


def stop_slomo(bool_force_stop=False):
   """Calls end_slomo"""
   if str_current_trigger:
      if bool_force_stop or dict_triggers[str_current_trigger]['interruptible']:
         end_slomo(bool_check_bots=str_current_trigger <> 'only_bots')


def add_trigger(str_trigger, float_duration, float_timescale, int_interruptible=1, str_sound='', int_downloadable=1, float_volume=-1):
   """Adds the trigger name and settings"""
   global dict_triggers

   if str_sound == '':
      int_downloadable = 0

   dict_triggers[str_trigger] = {'duration':float_duration, 'timescale':float_timescale, 'interruptible':int_interruptible, 'sound':str_sound, 'downloadable':int_downloadable, 'volume':float_volume}


def remove_trigger(str_trigger):
   """Removes the trigger name"""
   global dict_triggers

   if dict_triggers.has_key(str_trigger):
      if str_current_trigger == str_trigger:
         end_slomo(bool_check_bots=str_trigger <> 'only_bots')
      del dict_triggers[str_trigger]


def trigger_cmd():
   """
   slomomod_add <trigger> <duration> <timescale> [interruptible] [sound] [force sound download] [volume]
   slomomod_remove <trigger>
   Adds the trigger with the supplied arguments
   Displays a warning if the trigger was not recognized
   """
   str_command = es.getargv(0).lower()
   int_arg_count = es.getargc()
   if int_arg_count in {'slomomod_add':(4, 5, 6, 7, 8), 'slomomod_remove':(2,)}[str_command]:
      str_trigger = es.getargv(1).lower()
      if str_command == 'slomomod_add':
         tuple_arg_order = (float, float, int, str, int, float)
         list_args = [str_trigger]
         for int_x in range(2, int_arg_count):
            list_args.append(tuple_arg_order[int_x - 2](es.getargv(int_x)))
         add_trigger(*list_args)

         int_underscore_position = str_trigger.find('_')
         if str_trigger not in ('death_onestanding', 'death_lastteam', 'death_lasthuman', 'humandeath_headshot', 'humankill_headshot', 'death_headshot', 'humandeath', 'humankill', 'death', 'bomb_exploded', 'bomb_defused', 'hostage_killed', 'all_hostages_rescued', 'hostage_rescued', 'roundend_timelimit', 'roundend', 'only_bots') and int_underscore_position <> -1:
            if str_trigger[0:int_underscore_position] in ('death', 'humankill', 'humandeath'):
               str_weapon = str_trigger[int_underscore_position + 1:]
               int_str_len = len(str_weapon)
               if str_weapon[int_str_len - 9:int_str_len] == '_headshot':
                  str_weapon = str_weapon[0:int_str_len - 9]
                  bool_headshot = True
               else:
                  bool_headshot = False

               if str_weapon not in tuple_weapon_tags and not dict_weapons.has_key(str_weapon):
                  es.dbgmsg(0, 'Warning: Trigger \"%s\" was added but weapon \"%s\" was not recognized.' % (str_trigger, str_weapon))

               elif bool_headshot and str_weapon in ('hegrenade', 'knife', 'team', 'team_suicide', 'other'):
                  es.dbgmsg(0, 'Warning: Trigger \"%s\" was added but weapon \"%s\" does not support headshots.' % (str_trigger, str_weapon))

            else:
               es.dbgmsg(0, 'Warning: Trigger \"%s\" was added but not recognized.' % str_trigger)

      else:
         remove_trigger(str_trigger)

   else:
      es.dbgmsg(0, 'Syntax: ' + {'slomomod_add':'slomomod_add <trigger> <duration> <timescale> [interruptible] [sound] [force sound download] [volume]', 'slomomod_remove':'slomomod_remove <trigger>'}[str_command])


def start_cmd():
   """
   slomomod_start <duration> <timescale> [interruptible] [sound] [force sound download] [volume]
   Calls add_trigger and adds the "custom" trigger with the supplied arguments as settings
   Triggers the "custom" timescale
   """
   global dict_triggers

   int_arg_count = es.getargc()
   if int_arg_count in (3, 4, 5, 6, 7):
      tuple_arg_order = (float, float, int, str, int, float)
      list_args = ['custom']
      for int_x in range(1, int_arg_count):
         list_args.append(tuple_arg_order[int_x - 1](es.getargv(int_x)))
      add_trigger(*list_args)

      trigger_slomo('custom')

   else:
      es.dbgmsg(0, 'Syntax: slomomod_start <duration> <timescale> [interruptible] [sound] [force sound download] [volume]')


def end_cmd():
   """Calls stop_slomo with the argument translated to a boolean value"""
   if es.getargc() > 1:
      int_force_end = int(es.getargv(1))
   else:
      int_force_end = 0

   stop_slomo(bool(int_force_end))


def tracers_on():
   """
   Sets eventscripts_noisy
   Sets tracer status
   """
   global int_tracers_status

   if not int_tracers_status:
      es.doblock('corelib/noisy_on')
      sv_noisy = es.ServerVar('eventscripts_noisy')
      if int(sv_noisy):
         int_tracers_status = 1
      else:
         sv_noisy.set(1)
         int_tracers_status = 2


def tracers_off():
   """
   Removes eventscripts_noisy
   Removes tracer status and data
   """
   global int_tracers_status
   global dict_tracer_data

   if int_tracers_status:
      if int_tracers_status == 2:
         es.ServerVar('eventscripts_noisy').set(0)
      else:
         es.doblock('corelib/noisy_off')
      int_tracers_status = 0

   dict_tracer_data.clear()


def load_events():
   """Loads the .res file for the events"""
   if os.path.isfile(es.getAddonPath('slomomod') + '/slomomod.res'):
      es.loadevents('declare', 'addons/eventscripts/slomomod/slomomod.res')
   else:
      if not os.path.isfile(es.getAddonPath('slomomod') + 'es_slomomod_events_db.txt'):
         es.keygroupcreate('slomomod_events')
         for str_event in dict_events:
            es.keycreate('slomomod_events', str_event)
            for str_event_var in dict_events[str_event]:
               es.keysetvalue('slomomod_events', str_event, str_event_var, dict_events[str_event][str_event_var])
         es.keygroupsave('slomomod_events', '|slomomod')
         es.keygroupdelete('slomomod_events')
      es.loadevents('declare', 'addons/eventscripts/slomomod/es_slomomod_events_db.txt')
      es.dbgmsg(0, 'SloMoMod: Unable to load slomomod.res! Please ensure it is in the ./slomomod/ directory.')